import { Request } from "express";
import { v4 as uuidv4 } from "uuid";
import User, { UserPayload } from "../models/user.modal";
import { getCompanyByOptions } from "./company.service";
import { generateToken, verifyToken } from "./jwtToken";

export const createUserToken = async (token: string, companyName: string) => {
  try {
    const company = await getCompanyByOptions({ name: companyName });
    if (!company) throw "404:Company not found";

    const user = verifyToken(token, company.secretKey);
    if (!user) throw "0001:User not found";

    const userModel = await createUserModel(user);
    return userModel;
  } catch (error) {
    if (error instanceof Error) throw error.message;

    throw new Error((error as string) || "500:Failed to create user token");
  }
};

export const createJWTToken = async (body: Request["body"]) => {
  try {
    const { name, amount, lobby, companyName } = body;

    const company = await getCompanyByOptions({ name: companyName });
    if (!company) throw "404:Company not found";

    const token = generateToken({ name, amount, lobby }, company.secretKey);
    return token;
  } catch (error) {
    if (error instanceof Error) throw error.message;

    throw new Error((error as string) || "500:Failed to create JWT token");
  }
};

export const createUserModel = async (user: UserPayload) => {
  try {
    const { iat, exp, ...userPayload } = user;
    const token = uuidv4();

    const newUser = {
      ...userPayload,
      token: token,
      count: 0,
    };

    const userModel = await User.create(newUser);
    return userModel;
  } catch (error) {
    throw new Error((error as string) || "500:Failed to create user model");
  }
};
